﻿############################################################# 
<#
Copyright (c) 2018 Ninaronline.co.uk - Amir Joseph Sayes

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
#>
############################################################


Set-ExecutionPolicy -ExecutionPolicy bypass -Scope Process


#Load libreries to support Test-KeyPress function
#[void][System.Reflection.Assembly]::LoadWithPartialName("System.Drawing")
#[void][System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
add-type -AssemblyName System.Windows.Forms -Verbose
add-type -AssemblyName System.Drawing -verbose
[void][System.Reflection.Assembly]::LoadWithPartialName("System.Drawing") 
[void][System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") 


##Functions

Function LogFile {
    param(
        [string]$Message,
        [string]$Type,
        [switch]$WriteToFile = $true,
        [string]$FileLocation = $LocallLogFolder,
        [switch]$writeToHost = $true
    )
    
    $tDateTime = get-date -format yyyy-MM-dd-HH-mm-ss

    $tString = "$tDateTime - $Message"

    if($writeToHost) { Write-host $tString -ForegroundColor Green}

    if($WriteToFile) { $tString | Out-File -Append -FilePath $FileLocation\EventLogs.csv 
    }
}



Function Get-DiskSpace([string] $Computer)
{
        try { $HD = Get-WmiObject Win32_LogicalDisk -ComputerName $Computer -Filter "DeviceID='C:'" -ErrorAction Stop } 
        catch { Write-Warning "Unable to connect to $Computer. $_"; continue}
        if(!$HD) { throw }
        $FreeSpace = [math]::round(($HD.FreeSpace/1gb),2)
        return $FreeSpace
}


Function Kill-process {
 [cmdletbinding()]
param(
  $ComputerName=$env:COMPUTERNAME,
  [parameter(Mandatory=$true)]
  $ProcessName
)
$Processes = Get-WmiObject -Class Win32_Process -ComputerName $ComputerName -Filter "name='$ProcessName'"
 
    foreach ($process in $processes) {
      $returnval = $process.terminate()
      $processid = $process.handle
 
        if($returnval.returnvalue -eq 0) {
          write-host "The process $ProcessName `($processid`) terminated successfully"
        }
        else {
          write-host "The process $ProcessName `($processid`) termination has some problems"
        }
    }
}


function is-Numeric ($x) {
    $x2 = 0
    $isNum = [System.Int32]::TryParse($x, [ref]$x2)
    return $isNum
}



function Test-KeyPress
{
    <#
        .SYNOPSIS
        Checks to see if a key or keys are currently pressed.

        .DESCRIPTION
        Checks to see if a key or keys are currently pressed. If all specified keys are pressed then will return true, but if 
        any of the specified keys are not pressed, false will be returned.
        https://gallery.technet.microsoft.com/scriptcenter/Check-for-Key-Presses-with-7349aadc/view/Discussions#content

        .PARAMETER Keys
        Specifies the key(s) to check for. These must be of type "System.Windows.Forms.Keys"

        .EXAMPLE
        Test-KeyPress -Keys ControlKey

        Check to see if the Ctrl key is pressed

        .EXAMPLE
        Test-KeyPress -Keys ControlKey,Shift

        Test if Ctrl and Shift are pressed simultaneously (a chord)

        .LINK
        Uses the Windows API method GetAsyncKeyState to test for keypresses
        http://www.pinvoke.net/default.aspx/user32.GetAsyncKeyState

        The above method accepts values of type "system.windows.forms.keys"
        https://msdn.microsoft.com/en-us/library/system.windows.forms.keys(v=vs.110).aspx

        .LINK
        http://powershell.com/cs/blogs/tips/archive/2015/12/08/detecting-key-presses-across-applications.aspx

        .INPUTS
        System.Windows.Forms.Keys

        .OUTPUTS
        System.Boolean
    #>
    
    [CmdletBinding()]
    param
    (
        [Parameter(Mandatory = $true)]
        [ValidateNotNullOrEmpty()]
        [System.Windows.Forms.Keys[]]
        $Keys
    )   
    

    # use the User32 API to define a keypress datatype
    $Signature = @'
    [DllImport("user32.dll", CharSet=CharSet.Auto, ExactSpelling=true)] 
    public static extern short GetAsyncKeyState(int virtualKeyCode); 
'@
    $API = Add-Type -MemberDefinition $Signature -Name 'Keypress' -Namespace Keytest -PassThru
    
    # test if each key in the collection is pressed
    $Result = foreach ($Key in $Keys)
    {
        [bool]($API::GetAsyncKeyState($Key) -eq -32767)
    }
    
    # if all are pressed, return true, if any are not pressed, return false
    $Result -notcontains $false
}
 #####End Of Functions ####
 
 #####Variables ###

$LocalTempPath = "c:\temp\ProcMon_Remote"
$Procmon = 'procmon.exe'
$AdminShare = 'c$'
$LocalDrive = 'c:'
$psexecPath = "$LocalTempPath\PsExec.exe"
$procmonPath = "$LocalTempPath\$Procmon"
$TargetFolder = "temp\ProcMon_Remote"
[string]$TheDate = Get-Date -Format "yyyy-MM-dd HHmmss"
#$LocallLogFolder = "$LocalTempPath\$($ComputerName)"
$ProcMonLogFileSize = 0
$procmonCaptureFile = "Caputer_writes_ProcmonConfiguration.pmc"
$DiskInfoAll = @()

## End of Variables### 

 ##Pre-requsits###
 #Checking if temp folder was created on the machine running the script, if not create it.
 If (!(Test-Path $LocalTempPath)) {New-Item -Path $LocalTempPath -ItemType Directory  -Force}

#Gettting the location of the script
if ($myInvocation.MyCommand.Path -ne $null) { $scriptPath = Split-Path $myInvocation.MyCommand.Path
} else { $scriptPath = (Get-Location).Path} #If the script isn't saved, use the current location

#Copy files to local machine from Script path
try { Copy-Item $scriptPath\* $LocalTempPath -Force -ErrorAction Stop } catch { Write-Warning $_.Exception; break }   
cls
$ComputerName = Read-Host "Please enter the computer name that you would like to monitor:" -


write-host "**** ProcMon Remote V1 ****" -BackgroundColor Black -ForegroundColor Yellow
sleep -Seconds 1
LogFile "The script is running from: $scriptPath" -WriteToFile:$false
sleep -Seconds 1
LogFile "Local Temp path on the machine running the script is: $LocalTempPath" -WriteToFile:$false 
sleep -Seconds 1
    
Logfile "Verifing that the target computer responds to ping..." -WriteToFile:$false 
if (!(Test-Connection $ComputerName -ErrorAction Stop)) { 
    Logfile "Cannot ping $ComputerName" -WriteToFile $false ; break 
} else { 
    Logfile "$Computername is pingable! Creating a local subfolder for capturing if not already created..." -WriteToFile:$false 
    Start-Sleep -Seconds 2
    $LocallLogFolder = "$LocalTempPath\$($ComputerName)"
    if (!(Test-Path $LocallLogFolder)) {New-Item -path "$LocallLogFolder" -type directory -Verbose -Force; Logfile "$LocallLogFolder Has been created..."}
    Start-Sleep -Seconds 2
}

LogFile "Check that we have all supporting files on local machine" -ForegroundColor Green      
if (!(Test-Path $psexecPath -ErrorAction Stop)) { Logfile "Cannot find $psexecPath"; break } else {Logfile "$psexecPath  Exists"}
if (!(Test-Path $procmonPath -ErrorAction Stop)) { Logfile "Cannot find $procmonPath"; break } else {Logfile "$procmonPath Exists"}
if (!(Test-Path $LocalTempPath\$procmonCaptureFile -ErrorAction Stop)) { Logfile "Cannot find $LocalTempPath\$procmonCaptureFile"; break } else {Logfile "$LocalTempPath\$procmonCaptureFile Exists"}


#Check that the target computer have a folder under c:\temp to copy the files
If (!(Test-Path "\\$ComputerName\$AdminShare\$TargetFolder")) {logfile "Path \\$ComputerName\$AdminShare\$TargetFolder doesn't exit... Creating..."; New-Item -Path "\\$ComputerName\$AdminShare\$TargetFolder" -ItemType Directory  -Force}
Start-Sleep -Seconds 2

###End of Pre-requists ####  

##### Dialougs

#How many ProcMon captures would you like to take? Type: "All" to capture indefinitely or a certain number  
$ModeChoice = $null
Do {
$ModeChoice  =  Read-Host -Prompt  "How many ProcMon captures would you like to take? Type: '0' to capture indefinitely or enter a number between 1 and 50"
If (Is-Numeric $ModeChoice -eq $true) { try {$ModeChoice = [convert]::ToInt32($ModeChoice)} catch {Logfile "The input is not valid... Exiting..."; Start-Sleep 3}} else {Remove-Variable ModeChoice}
} until ($ModeChoice -eq 0 -or ($ModeChoice -gt 0 -and $ModeChoice -le 50))
If ($ModeChoice -eq 0) {LogFile "Indifinate capture mode was chosen"; $captureMode = 0} 
Elseif ($ModeChoice -gt 0 -and $ModeChoice -le 50 ) {LogFile "$ModeChoice capture(s) chosen"; $captureMode = $ModeChoice}
else {LogFile "Invalid entry. Exiting..."; Start-Sleep -Seconds 3 ; break}


# What size [MB] each caputre file should have? Default is [5 MB] 
$Sizechoice = $null
Do {
$Sizechoice = Read-Host -Prompt "What is the max file caputer size at which a new capture is started and the current capture is downloaded to your machine. Min is 5 MB, Max is 100 MB"
If (Is-Numeric $Sizechoice -eq $true) {$Sizechoice = [convert]::ToDouble($Sizechoice)} else {Remove-Variable Sizechoice}
} Until ($Sizechoice -ge 5 -and $Sizechoice -le 100)

if ($Sizechoice -ge 5 -and $Sizechoice -le 100) {Logfile "Capture size chosen is $Sizechoice"; $CaptureSize = $Sizechoice}
Else {LogFile "Invalid entry. Exiting..."; Start-Sleep -Seconds 3 ; break} 
 

#What would you like to capture? 
#Disk Read/Writes - Using a pre-configured filter file (pmc)
#Registry Keys - Using a pre-configured filter file (pmc)
#Everything - no configuration file 
    
Write-Host "What would you like to capture?"
$options = "Disk Read/Writes - Using a pre-configured filter file (pmc)","Registry Keys - Using a pre-configured filter file (pmc)","Everything - no configuration file","Exit Script"
$options[3]
$i=0 
for ($i=0; $i -lt $options.Count; $i++ ) {
    Write-Host ($($i + 1).ToString() + ":") $options[$i]
}
$choice = $null
do {
    $choice = Read-Host -Prompt "Enter the option number"
} until (($choice -gt 0 -and $choice -le $options.Count)) 
    
Logfile "$($options[$choice-1]) was chosen"
$WhatToCapture = $choice


if ($WhatToCapture -eq 1) {$procmonCaptureFile = "Caputer_writes_ProcmonConfiguration.pmc"}
elseif ($WhatToCapture -eq 2) {$procmonCaptureFile = "Caputer_RegKeys_Configuration.pmc"}
elseif ($WhatToCapture -eq 3) {$procmonCaptureFile = "Caputer_Everything_Configuration.pmc"}
elseif ($WhatToCapture -eq 4) {Write-host "Exiting Script..."; Start-Sleep -Seconds 3; break}
else {Write-host "Error - Exiting Script..."; break}

#verify that the config file is present on the local machine. 
if (!(test-path $LocalTempPath\$procmonCaptureFile)) {Write-Host "Configuration file $LocalTempPath\$procmonCaptureFile was not found";break }
#rest the $Choice variable
$choice = $null

#Monitor Hard Disk free sapce? 
Do {
$MonitorDisk = Read-Host -Prompt "Would you like to Monitor Disk Space while capturing? Type 'Y' or 'N'"
} until ($MonitorDisk -eq "Y" -or $MonitorDisk -eq "N")
Logfile "You answered $MonitorDisk for monitoring Disk Space"
 

##### End of Dialougs

Logfile "Kill any pre-existing procmon processes on the target machine..."
Get-Process -ComputerName $ComputerName -Name procmon* | Select processname | % {$tProcessname = $_.ProcessName + ".exe"; LogFile "Killing any existing procmon processes: $tProcessname"; Kill-process -ComputerName $ComputerName -ProcessName $tProcessname}

#Copying Process monitor to the target system  so we can execute it locally with psexec.
LogFile "Copying Process monitor to the target system so we can execute it locally with psexec."
    try {Copy-Item $procmonPath "\\$ComputerName\$AdminShare\$TargetFolder" -Force -Verbose -ErrorAction Stop } catch { Write-Warning $_.Exception; break }    
    try { Copy-Item $LocalTempPath\*.pmc "\\$ComputerName\$AdminShare\$TargetFolder" -Force -Verbose -ErrorAction Stop } catch { Write-Warning $_.Exception; break }            
Start-Sleep -Seconds 2

#get the current user info and a list of processes and save them in the log folder
#Logfile "Getting a list of processes running on the target machine..."
#Get-WhatsGoingOn -ComputerName $computerName -tLocallLogFolder $LocallLogFolder -TargerTempFolder "$LocalDrive\$TargetFolder" -psexecPath $psexecPath -ErrorAction SilentlyContinue

# Process monitor must be launched as a separate process otherwise the sleep and terminate commands below would never execute and fill the disk

If ($ModeChoice -eq 0) { #capture indefinitley and wait for exit key         
    Logfile "Starting process monitor on $ComputerName"
    Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /accepteula /loadconfig $LocalTempPath\$procmonCaptureFile /backingfile $LocalTempPath\$ComputerName /quiet" -PassThru | out-null
    Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /accepteula /WaitForIdle" -PassThru | out-null
    Start-Sleep -Seconds 2        
        Do {
            $Break = $false
            If ($MonitorDisk -eq "Y") {$disk = Get-WmiObject Win32_LogicalDisk -ComputerName $ComputerName -Filter "DeviceID <> 'A:'"} 

            if ($ProcMonLogFileSize -and $ProcMonLogFileSize -gt $CaptureSize){
                LogFile "Log file is full. Terminating process monitor process on $ComputerName and Starting a new capture with a new file..."
                $Discard = & $psexecPath -s -d \\$ComputerName "$LocalDrive\$TargetFolder\$Procmon" /terminate 2>&1; $Output | select -skip 1 | out-string            
                Start-Sleep 10

                Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /PagingFile /NoConnect /Minimized /Quiet" -PassThru | out-null
                #start-Process -FilePath $psexecPath -ArgumentList "\\$ComputerName -s -i 0 $LocalDrive\$TargetFolder\start_paging.cmd" -PassThru | Out-null 
                Start-Sleep 10 
                $Discard = & $psexecPath -s -d \\$ComputerName "$LocalDrive\$TargetFolder\$Procmon" /terminate 2>&1; $Output | select -skip 1 | out-string
                #start-Process -FilePath $psexecPath -ArgumentList "\\$ComputerName -s -i 0 $LocalDrive\$TargetFolder\Stop_logging.cmd" -PassThru | Out-null         
                Start-Sleep 5
                ##Rename file
                [string]$TheDate = Get-Date -Format "yyyy-MM-dd HHmmss"
                try { Rename-Item -path "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.pml" -NewName "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName$TheDate.pml" -Force -Verbose -ErrorAction Stop} catch { $_ ; }
                    LogFile "Copy process monitor file to local machine for analysis"
                try {             
                    if (!(Test-Path $LocallLogFolder)) {New-Item -path "$LocallLogFolder" -type directory -Verbose -Force}
                    Copy-Item "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName$TheDate.pml" $LocallLogFolder\$ComputerName$TheDate.pml  -Force -Verbose -ErrorAction Stop
                    }  
                catch { $_ ; } 
                $ReCapture = $true
                Start-Sleep 5
                }
                ### to recapture when the condition is met
                if ($ReCapture -eq $true) {
                    Logfile "Starting process monitor on $ComputerName"
                    Start-Sleep -Seconds 5
                    Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /accepteula /loadconfig $LocalTempPath\$procmonCaptureFile /backingfile $LocalTempPath\$ComputerName /quiet" -PassThru
                    Start-Sleep -Seconds 5
                    Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /accepteula /WaitForIdle" -PassThru 
                    Start-Sleep -Seconds 5
                    $ReCapture = $false   
                }
                ###Sometimes it takes few seconds for the pml file to be created so this checks if the file is actually there before calculating size infomation, etc... 
                while (!(Test-Path -Path "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.PML"))
                {
                    Start-Sleep -Seconds 2    
                    Logfile "Waiting for caputer Files to be created...Press and hold F12 to stop the script"
                    $Break = Test-KeyPress -Keys "F12"
                    if ($Break -eq $true) {
                        Logfile "Kill any pre-existing procmon processes on the target machine..."
                        Get-Process -ComputerName $ComputerName -Name procmon* | Select processname | % {$tProcessname = $_.ProcessName + ".exe"; LogFile "Killing any existing procmon processes: $tProcessname"; Kill-process -ComputerName $ComputerName -ProcessName $tProcessname}
                        Break
                    } 
                    else {$Break = $false}
                }
        
                ###
                Start-Sleep -Seconds 3
                $ProcMonLogFileSize = (Get-ChildItem -Path \\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.PML  | measure -Property length -Sum).Sum /1mb
                If ($MonitorDisk -eq "Y") { #do the below if disk inforamtion was chosen to be displayed
                    foreach ($Diskdrive in $Disk) {
                        $DiskInfo = "" | select DriveFreePrecentage, DiskName
                        $DiskInfo.DiskName = $Diskdrive.DeviceID                 
                        $DiskInfo.DriveFreePrecentage = [math]::Round($Diskdrive.FreeSpace/$Diskdrive.Size*100,2)
                        $DiskInfoAll += $DiskInfo
                    }    
                    #Loop inside diskinfo drives and disply the info
                    0..($DiskInfoAll.Count-1) | % {LogFile "Drive: $($DiskInfoAll[$_].DiskName) - FreePrecentage: $($DiskInfoAll[$_].DriveFreePrecentage)%"}                
                    $DiskInfoAll = @()                    
                } 
                Logfile "Writing to file $computername.pml - File size: $ProcMonLogFileSize MB - Press and hold F12 to stop capturing..."
                
                $Break = Test-KeyPress -Keys "F12"
                #$Break
                Start-Sleep -Seconds 3        
        }until ($Break -eq $true)
                #F12 is pressed - stop capturing and download the captured file
                LogFile "F12 was pressed. Stopping the capture and copying down the capture file... "
                $Discard = & $psexecPath -s -d \\$ComputerName "$LocalDrive\$TargetFolder\$Procmon" /terminate 2>&1; $Output | select -skip 1 | out-string            
                Start-Sleep 10

                Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /PagingFile /NoConnect /Minimized /Quiet" -PassThru | out-null                
                Start-Sleep 10 

                $Discard = & $psexecPath -s -d \\$ComputerName "$LocalDrive\$TargetFolder\$Procmon" /terminate 2>&1; $Output | select -skip 1 | out-string                
                Start-Sleep 5

                Logfile "Kill any pre-existing procmon processes on the target machine..."
                Get-Process -ComputerName $ComputerName -Name procmon* | Select processname | % {$tProcessname = $_.ProcessName + ".exe"; LogFile "Killing any existing procmon processes: $tProcessname"; Kill-process -ComputerName $ComputerName -ProcessName $tProcessname}

                ##Rename file
                [string]$TheDate = Get-Date -Format "yyyy-MM-dd HHmmss"
                try { 
                    Rename-Item -path "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.pml" -NewName "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName$TheDate.pml" -Force -Verbose -ErrorAction Stop
                    } 
                catch { $_ ; }

                    LogFile "Copy process monitor file to local machine for analysis"
                try {             
                    if (!(Test-Path $LocallLogFolder)) {New-Item -path "$LocallLogFolder" -type directory -Verbose -Force}
                    Copy-Item "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName$TheDate.pml" $LocallLogFolder\$ComputerName$TheDate.pml  -Force -Verbose -ErrorAction Stop
                    }  
                catch { $_ ; } 
                start-sleep 5                 

    } 
else { #do loop for number of captures 
    
    for ([int]$i=1;$i -le $ModeChoice;$i++ ) {
        Logfile "Starting process monitor on $ComputerName - Capture $i of $ModeChoice"
        Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /accepteula /loadconfig $LocalTempPath\$procmonCaptureFile /backingfile $LocalTempPath\$ComputerName /quiet" -PassThru | out-null
        Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /accepteula /WaitForIdle" -PassThru | out-null
        Start-Sleep -Seconds 5  
        
        #Sometimes it takes few seconds for the pml file to be created so this checks if the file is actually there before calculating size infomation, etc... 
        while (!(Test-Path -Path "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.PML"))
            {
                Start-Sleep -Seconds 2    
                Logfile "Waiting for caputer Files to be created...Press and hold F12 to stop the script"
                $Break = Test-KeyPress -Keys "F12"
                if ($Break -eq $true) {
                    Logfile "Kill any pre-existing procmon processes on the target machine..."
                    Get-Process -ComputerName $ComputerName -Name procmon* | Select processname | % {$tProcessname = $_.ProcessName + ".exe"; LogFile "Killing any existing procmon processes: $tProcessname"; Kill-process -ComputerName $ComputerName -ProcessName $tProcessname}
                    Break
                } 
                else 
                {$Break = $false}
            }
                  
        
        #get Disk info if user has chosen so and show it
        Do {            
            $Break = $false
            $ProcMonLogFileSize = (Get-ChildItem -Path \\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.PML  | measure -Property length -Sum).Sum /1mb
            If ($MonitorDisk -eq "Y") {
                #do the below if disk inforamtion was chosen to be displayed
                $disk = Get-WmiObject Win32_LogicalDisk -ComputerName $ComputerName -Filter "DeviceID <> 'A:'"
                foreach ($Diskdrive in $Disk) {
                    $DiskInfo = "" | select DriveFreePrecentage, DiskName
                    $DiskInfo.DiskName = $Diskdrive.DeviceID                 
                    $DiskInfo.DriveFreePrecentage = [math]::Round($Diskdrive.FreeSpace/$Diskdrive.Size*100,2)
                    $DiskInfoAll += $DiskInfo
                    }
                #Loop inside diskinfo drives and disply the info
                0..($DiskInfoAll.Count-1) | % {LogFile "Drive: $($DiskInfoAll[$_].DiskName) - FreePrecentage: $($DiskInfoAll[$_].DriveFreePrecentage)%"}                
                $DiskInfoAll = @()
            } 
            
            Logfile "Writing to file $computername.pml - File size: $ProcMonLogFileSize MB - Press and hold F12 to stop capturing..."            
            
            #Listen to F12 and break if pressed
            $Break = Test-KeyPress -Keys "F12"            
            Start-Sleep -Seconds 3        

        } until (($ProcMonLogFileSize -gt $CaptureSize) -or ($Break -eq $true))       
        
            #If the file size has reached or if F12 is pressed - stop capturing and download the captured file
            LogFile "File size has reached or F12 was pressed - Stopping the capture and copying down the capture file... "
            $Discard = & $psexecPath -s -d \\$ComputerName "$LocalDrive\$TargetFolder\$Procmon" /terminate 2>&1; $Output | select -skip 1 | out-string            
            Start-Sleep 10

            Start-Process -FilePath $psexecPath -ArgumentList "-s -d \\$ComputerName $LocalDrive\$TargetFolder\$Procmon /PagingFile /NoConnect /Minimized /Quiet" -PassThru | out-null            
            Start-Sleep 5

            $Discard = & $psexecPath -s -d \\$ComputerName "$LocalDrive\$TargetFolder\$Procmon" /terminate 2>&1; $Output | select -skip 1 | out-string
            
            Start-Sleep 10
            
            Logfile "Kill any pre-existing procmon processes on the target machine..."
            Get-Process -ComputerName $ComputerName -Name procmon* | Select processname | % {$tProcessname = $_.ProcessName + ".exe"; LogFile "Killing any existing procmon processes: $tProcessname"; Kill-process -ComputerName $ComputerName -ProcessName $tProcessname} 
            
            ##Rename file
            [string]$TheDate = Get-Date -Format "yyyy-MM-dd HHmmss"
            try { 
                Rename-Item -path "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName.pml" -NewName "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName$TheDate.pml" -Force -Verbose -ErrorAction Stop
                } 
            catch { $_ ; }

                LogFile "Copy process monitor file to local machine for analysis"
            try {             
                if (!(Test-Path $LocallLogFolder)) {New-Item -path "$LocallLogFolder" -type directory -Verbose -Force}
                Copy-Item "\\$ComputerName\$AdminShare\$TargetFolder\$ComputerName$TheDate.pml" $LocallLogFolder\$ComputerName$TheDate.pml  -Force -Verbose -ErrorAction Stop
                }  
            catch { $_ ; }
            start-sleep 5 
            
            #If F12 was pressed stop the loop
            If ($Break -eq $true) {$Break = $false; break}
            
        } #end of for

}  #end of else